# -*- coding: utf-8 -*-
from typing import Dict, Optional

from abeja.common.api_client import BaseAPIClient


class APIClient(BaseAPIClient):
    """A low-level client for Model API

    .. code-block:: python

       from abeja.models import APIClient

       api_client = APIClient()
    """

    def __init__(self, credential: Optional[Dict[str, str]]=None) -> None:
        super().__init__(credential=credential)

    def create_model(self, organization_id: str, name: str, description: str) -> dict:
        """create a model

        API reference: POST /organizations/<organization_id>/models/

        Request Syntax:
            .. code-block:: python

               organization_id = "1111111111111"
               model_name = "model_name"
               model_description = "this is description of the model"

               response = api_client.create_model(organization_id, model_name, model_description)

        Params:
            - **organization_id** (str): organization identifier
            - **name** (str): model name
            - **description** (str): model description

        Return type:
            dict
        Returns:
            Response Syntax:
                .. code-block:: json

                   {
                       "model_id": "1111111111111",
                       "name": "model_name",
                       "description": "this is description of the model",
                       "created_at": "2018-01-01T00:00:00.00000Z",
                       "modified_at": "2018-01-01T00:00:00.00000Z",
                       "versions": []
                   }

            Response Structure:
                - **model_id** (str) : model id
                - **name** (str) : model name
                - **description** (str) : model description
                - **versions** (list) :

        Raises:
            - BadRequest: the resource already exists or parameters is insufficient or invalid.
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        payload = {
            'name': name,
            'description': description
        }
        path = '/organizations/{}/models'.format(organization_id)
        return self._connection.api_request(method='POST', path=path, json=payload)

    def get_model(self, organization_id: str, model_id: str) -> dict:
        """get a model

        API reference: GET /organizations/<organization_id>/models/<model_id>

        Request Syntax:
            .. code-block:: python

               response = api_client.get_model(organization_id='1111111111111', model_id='1111111111111')

        Params:
            - **organization_id** (str): organization_id
            - **model_id** (str): model_id of the requested model

        Return type:
            dict

        Returns:
            Response Syntax:
                .. code-block:: json

                   {
                       "model_id": "1111111111111",
                       "name": "model_name",
                       "description": "this is description of the model",
                       "created_at": "2018-01-01T00:00:00.00000Z",
                       "modified_at": "2018-01-01T00:00:00.00000Z",
                       "versions": []
                   }

            Response Structure:
                - **model_id** (str) : model id
                - **name** (str) : model name
                - **description** (str) : model description
                - **versions** (list) :

        Raises:
          - NotFound: model not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """

        path = '/organizations/{}/models/{}'.format(organization_id, model_id)
        return self._connection.api_request(method='GET', path=path)

    def get_models(self, organization_id: str) -> dict:
        """Get models entries

        API reference: GET /organizations/<organization_id>/models/

        Request syntax:
            .. code-block:: python

               response = api_client.list_models(organization_id='1102940376065')

        Params:
            - **organization_id** (str): organization_id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "entries": [
                           {
                               "model_id": "1111111111111",
                               "name": "model_name",
                               "description": "this is description of the model",
                               "created_at": "2018-01-01T00:00:00.00000Z",
                               "modified_at": "2018-01-01T00:00:00.00000Z",
                               "versions": []
                           }
                       ]
                   }

            Response Structure:
                - **entries** (list)
                    - (dict)
                        - **model_id** (str) : model id
                        - **name** (str) : model name
                        - **description** (str) : model description
                        - **versions** (list) :

        Raises:
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/models'.format(organization_id)
        return self._connection.api_request(method='GET', path=path)

    def delete_model(self, organization_id: str, model_id: str) -> dict:
        """delete a model

        API reference: DELETE /organizations/<organization_id>/model/<model_id>

        Request syntax:
            .. code-block:: python

               response = api_client.delete_model(organization_id='1111111111111', model_id='1111111111111')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **model_id** (str): model id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "message": "1111111111111 deleted"
                   }

            Response Structure:
                - **message** (str) :

        Raises:
          - NotFound: model not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/models/{}'.format(organization_id, model_id)
        return self._connection.api_request(method='DELETE', path=path)

    # model version
    def create_model_version(
            self, organization_id: str, model_id: str, version: str, handler: str,
            image: str, content_type: Optional[str]=None, job_definition_name: Optional[str]=None,
            training_job_id: Optional[str]=None) -> dict:
        """create a model version

        API reference: POST /organizations/<organization_id>/models/<model_id>/versions/

        Request syntax:
            .. code-block:: python

               model_id = '1111111111111',
               version = '0.0.1'
               handler = 'main:handler'
               image = 'abeja-inc/all-cpu:18.10'
               content_type = 'application/zip'

               response = api_client.create_model_version(
                   organization_id, model_id, version, handler, image, content_type
               )

        Params:
            - **organization_id** (str): organization id
            - **model_id** (str): model id
            - **version** (str): version
            - **handler** (str) : handler path
            - **content_type** (str) : Mime-type of content **[OPTIONAL]**
            - **job_definition_name** (str) : job definition name used in a model handler **[OPTIONAL]**
            - **training_job_id** (str) : job id **[OPTIONAL]**

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "model_id": "1111111111111",
                       "version": "0.0.1",
                       "version_id": "ver-abc1111111111111",
                       "handler": "main:handler",
                       "image": "abeja-inc/all-cpu:18.10",
                       "created_at": "2018-06-14T07:15:43.462664Z",
                       "modified_at": "2018-06-14T07:15:43.462824Z",
                       "job_definition_id": null,
                       "job_definition_version": null,
                       "training_job_id": null,
                       "upload_url": "https://abeja-model-api-src.s3.amazonaws.com/1462815098134/ver-/source.tgz?xxxxxx"
                   }

            Response Structure:
                - **model_id** (str) : model id
                - **version** (str) : version
                - **version_id** (str) : version id
                - **handler** (str) : handler path
                - **image** (str) : runtime enviornment
                - **job_definition_id** (str) : job definition id
                - **job_definition_version** (str) : job definition version
                - **training_job_id** (str) : job id
                - **upload_url** (str) : url to upload archived file. **archived file should be uploaded after creating model version using this upload_url**

        Raises:
          - BadRequest: specified model id does not exist or model version already exist, parameters is insufficient or invalid,
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        payload = {
            'version': version,
            'handler': handler,
            'image': image,
            'content_type': content_type,
        }
        if job_definition_name and training_job_id:
            payload.update({
                'job_definition_name': job_definition_name,
                'training_job_id': training_job_id,
            })
        # create model version
        path = '/organizations/{}/models/{}/versions'.format(organization_id, model_id)
        return self._connection.api_request(method='POST', path=path, json=payload)

    def get_model_version(self, organization_id: str, model_id: str, version_id: str) -> dict:
        """get a version of a model

        API reference: GET /organizations/<organization_id>/models/<model_id>/versions/<version_id>

        Request syntax:
            .. code-block:: python

               response = api_client.get_model_version(organization_id='1111111111111',
                                                       model_id='2222222222222',
                                                       version_id='ver-abc3333333333333')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **model_id** (str): model id
            - **version_id** (str): model version id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "model_id": "1111111111111",
                       "version_id": "ver-abc1111111111111",
                       "version": "0.0.1",
                       "handler": "main:handler",
                       "image": "abeja-inc/all-cpu:18.10",
                       "job_definition_id": null,
                       "job_definition_version": null,
                       "training_job_id": null,
                       "created_at": "2018-04-26T01:12:05.771403Z",
                       "modified_at": "2018-04-26T01:12:05.771533Z"
                   }

            Response Structure:
                - **model_id** (str) : model id
                - **version** (str) : version
                - **version_id** (str) : version id
                - **handler** (str) : handler path
                - **image** (str) : runtime enviornment
                - **job_definition_id** (str) : job definition id
                - **job_definition_version** (str) : job definition version
                - **training_job_id** (str) : job id

        Raises:
          - NotFound: model version not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/models/{}/versions/{}'.format(organization_id,
                                                                model_id,
                                                                version_id)
        return self._connection.api_request(method='GET', path=path)

    def get_model_versions(self, organization_id: str, model_id: str) -> dict:
        """Get version entries in a model

        API reference: GET /organizations/<organization_id>/models/<model_id>/versions/

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **model_id** (str): model id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "entries": [
                           {
                               "model_id": "1111111111111",
                               "version_id": "ver-abc1111111111111",
                               "version": "0.0.1",
                               "handler": "main:handler",
                               "image": "abeja-inc/all-cpu:18.10",
                               "job_definition_id": null,
                               "job_definition_version": null,
                               "training_job_id": null,
                               "created_at": "2018-04-26T01:12:05.771403Z",
                               "modified_at": "2018-04-26T01:12:05.771533Z"
                           }
                       ]
                   }

            Response Structure:
                - **entries** (list)
                    - (dict)
                        - **model_id** (str) : model id
                        - **version** (str) : version
                        - **version_id** (str) : version id
                        - **handler** (str) : handler path
                        - **image** (str) : runtime enviornment
                        - **job_definition_id** (str) : job definition id
                        - **job_definition_version** (str) : job definition version
                        - **training_job_id** (str) : job id

        Raises:
          - NotFound: model not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/models/{}/versions'.format(organization_id, model_id)
        return self._connection.api_request(method='GET', path=path)

    def delete_model_version(self, organization_id: str, model_id: str, version_id: str) -> None:
        """delete a version in a model

        API reference: DELETE /organizations/<organization_id>/models/<model_id>/versions/<version_id>

        Request syntax:
            .. code-block:: python

               api_client.delete_model_version(organization_id='1111111111111',
                                               model_id='1111111111111',
                                               version_id='ver-abc3333333333333')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **model_id** (str): model id
            - **version** (str): model version id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: json

                   {
                       "message": "ver-abc1111111111111 deleted"
                   }

            Response Structure:
                - **message** (str) :

        Raises:
          - NotFound: model version not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/models/{}/versions/{}'.format(organization_id, model_id, version_id)
        return self._connection.api_request(method='DELETE', path=path)
