from typing import Dict, Optional

from abeja.common.api_client import BaseAPIClient


class APIClient(BaseAPIClient):
    """A Low-Level client for Training API

    .. code-block:: python

       from abeja.train import APIClient

       api_client = APIClient()
    """

    def __init__(self, credential: Optional[Dict[str, str]]=None) -> None:
        super().__init__(credential=credential)

    def create_training_job_definition(self, organization_id: str, job_definition_name: str) -> dict:
        """create a training job definition

        API reference: POST /organizations/<organization_id>/training/definitions

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test"
                response = api_client.create_training_job_definition(organization_id,
                                                                     job_definition_name)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_id": "1443334816413",
                    "versions": [],
                    "organization_id": "1200123565071",
                    "modified_at": "2018-05-17T02:13:35.726812Z",
                    "created_at": "2018-05-17T02:13:35.726691Z",
                    "version_count": 0,
                    "name": "test"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        data = {'name': job_definition_name}
        path = '/organizations/{}/training/definitions/'.format(organization_id)
        return self._connection.api_request(method='POST', path=path, json=data)

    def get_training_job_definitions(self, organization_id: str) -> dict:
        """get training job definitions

        API reference: GET /organizations/<organization_id>/training/definitions

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                response = api_client.get_training_job_definitions(organization_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "entries": [
                        {
                            "version_count": 1,
                            "created_at": "2018-03-08T00:46:50.791787Z",
                            "organization_id": "1200123565071",
                            "versions": [
                                {
                                    "job_definition_version": 1,
                                    "user_parameters": {},
                                    "handler": "train:handler",
                                    "image": "abeja-inc/all-gpu:19.04",
                                    "modified_at": "2018-03-08T00:48:12.207883Z",
                                    "datasets": {
                                        "train": "1376063797251"
                                    },
                                    "created_at": "2018-03-08T00:48:12.132471Z",
                                    "job_definition_id": "1381349997580"
                                }
                            ],
                            "name": "test",
                            "modified_at": "2018-03-08T00:46:50.791946Z",
                            "job_definition_id": "1381349997580"
                        }
                    ]
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/'.format(organization_id)
        return self._connection.api_request(method='GET', path=path)

    def get_training_job_definition(self, organization_id: str, job_definition_name: str) -> dict:
        """get a training job definition.

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = 'test'
                response = api_client.get_training_job_definition(organization_id, job_definition_name)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "modified_at": "2018-05-17T02:13:35.726812Z",
                    "organization_id": "1200123565071",
                    "created_at": "2018-05-17T02:13:35.726691Z",
                    "job_definition_id": "1443334816413",
                    "name": "test",
                    "versions": [],
                    "version_count": 0
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}'.format(organization_id, job_definition_name)
        return self._connection.api_request(method='GET', path=path)

    def delete_training_job_definition(self, organization_id: str, job_definition_name: str) -> dict:
        """delete a training job definition.

        API reference: DELETE /organizations/<organization_id>/training/definitions/<job_definition_name>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = 'test'
                response = api_client.delete_training_job_definition(organization_id, job_definition_name)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "message": "test deleted"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}'.format(organization_id, job_definition_name)
        return self._connection.api_request(method='DELETE', path=path)

    def create_training_job_definition_version(
            self, organization_id: str, job_definition_name: str, params: dict=None) -> dict:
        """create a training job definition version.

        API reference: POST /organizations/<organization_id>/training/definitions/<job_definition_name>/versions

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = 'test_job_definition'
                params = {
                    "handler": "train:handler",
                    "datasets": {
                        "mnist": "1111111111111"
                    },
                    "image": "abeja-inc/all-gpu:19.04",
                    "source_code_base64": "....",
                    "user_parameters": {}
                }
                response = api_client.create_training_job_definition_version(organization_id, job_definition_name, params)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_version": 1,
                    "user_parameters": {},
                    "datasets": {
                        "mnist": "1111111111111"
                    },
                    "modified_at": "2018-05-17T12:34:46.344076Z",
                    "job_definition_id": "1443714239154",
                    "handler": "train:handler",
                    "created_at": "2018-05-17T12:34:46.296488Z",
                    "image": "abeja-inc/all-gpu:19.04"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/versions'.format(
            organization_id, job_definition_name)
        return self._connection.api_request(method='POST', path=path, json=params)

    def get_training_job_definition_versions(
            self, organization_id: str, job_definition_name: str) -> dict:
        """get training job definition versions.

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>/versions

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = 'test_job_definition'
                response = api_client.get_training_job_definition_versions(organization_id, job_definition_name)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "entries": [
                        {
                            "job_definition_version": 1,
                            "user_parameters": {},
                            "datasets": {
                                "mnist": "1111111111111"
                            },
                            "modified_at": "2018-05-17T12:34:46.344076Z",
                            "job_definition_id": "1443714239154",
                            "handler": "train:handler",
                            "created_at": "2018-05-17T12:34:46.296488Z",
                            "image": "abeja-inc/all-gpu:19.04"
                        }
                    ]
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/versions'.format(
            organization_id, job_definition_name)
        return self._connection.api_request(method='GET', path=path)

    def get_training_job_definition_version(
            self, organization_id: str, job_definition_name: str, version_id: int) -> dict:
        """get a training job definition version

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>/versions/<version_id>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                version_id = 1
                response = api_client.get_training_job_definition_version(organization_id, job_definition_name, version_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **version_id** (int): training job version

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_version": 1,
                    "user_parameters": {},
                    "datasets": {
                        "mnist": "1111111111111"
                    },
                    "modified_at": "2018-05-17T12:34:46.344076Z",
                    "job_definition_id": "1443714239154",
                    "handler": "train:handler",
                    "created_at": "2018-05-17T12:34:46.296488Z",
                    "image": "abeja-inc/all-gpu:19.04"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/versions/{}'.format(
            organization_id, job_definition_name, version_id)
        return self._connection.api_request(method='GET', path=path)

    def delete_training_job_definition_version(
            self, organization_id: str, job_definition_name: str, version_id: int) -> dict:
        """delete a training job definition version

        API reference: DELETE /organizations/<organization_id>/training/definitions/<job_definition_name>/versions/<version_id>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                version_id = 1
                response = api_client.delete_training_job_definition_version(organization_id, job_definition_name, version_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **version_id** (int): training job version

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_version": 1,
                    "user_parameters": {},
                    "datasets": {
                        "mnist": "1111111111111"
                    },
                    "modified_at": "2018-05-17T12:34:46.344076Z",
                    "job_definition_id": "1443714239154",
                    "handler": "train:handler",
                    "created_at": "2018-05-17T12:34:46.296488Z",
                    "image": "abeja-inc/all-gpu:19.04"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/versions/{}'.format(
            organization_id, job_definition_name, version_id)
        return self._connection.api_request(method='DELETE', path=path)

    def create_training_job(self, organization_id: str, job_definition_name: str,
                            version_id: int, user_parameters: dict=None) -> dict:
        """create a training job

        API reference: POST /organizations/<organization_id>/training/definitions/<job_definition_name>/versions/<version_id>/jobs

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                version_id = 1
                user_parameters = {
                    'BATCH_SIZE': 50
                }
                response = api_client.create_training_job_definition_version(organization_id,
                                                                             job_definition_name,
                                                                             version_id,
                                                                             user_parameters)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **version_id** (int): training job version
            - **user_parameters** (dict): user defined parameters set as environment variables.

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_id": "1443714239154",
                    "user_parameters": {},
                    "start_time": null,
                    "created_at": "2018-05-17T12:43:59.322367Z",
                    "job_definition_version": 1,
                    "completion_time": null,
                    "status": "Pending",
                    "modified_at": "2018-05-17T12:43:59.322673Z",
                    "training_job_id": "1443722127663",
                    "creator": {
                        "email": "test@abeja.asia",
                        "is_registered": true,
                        "created_at": "2017-05-26T01:38:46Z",
                        "id": "1128347408389",
                        "display_name": null,
                        "updated_at": "2018-01-04T03:02:12Z",
                        "role": "admin"
                    },
                    "description": null,
                    "statistics": null
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        if user_parameters is None:
            user_parameters = {}
        data = {'user_parameters': user_parameters}
        path = '/organizations/{}/training/definitions/{}/versions/{}/jobs'.format(
            organization_id, job_definition_name, version_id)
        return self._connection.api_request(method='POST', path=path, json=data)

    def get_training_jobs(
            self, organization_id: str, job_definition_name: str) -> dict:
        """get training jobs

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>/jobs

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                response = api_client.get_training_jobs(organization_id, job_definition_name)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "entries": [
                        {
                            "user_parameters": {},
                            "start_time": null,
                            "training_job_id": "1443722127663",
                            "created_at": "2018-05-17T12:43:59.322367Z",
                            "completion_time": null,
                            "id": "1443722127663",
                            "job_definition_version": 1,
                            "description": null,
                            "statistics": null,
                            "job_definition_id": "1443714239154",
                            "modified_at": "2018-05-17T12:43:59.322673Z",
                            "status": "Pending",
                            "creator": {
                                "email": "test@abeja.asia",
                                "created_at": "2017-05-26T01:38:46Z",
                                "id": "1128347408389",
                                "role": "admin",
                                "display_name": null,
                                "updated_at": "2018-01-04T03:02:12Z",
                                "is_registered": true
                            }
                        }
                    ],
                    "limit": 10,
                    "offset": 0,
                    "total": 1
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/jobs'.format(
            organization_id, job_definition_name)
        return self._connection.api_request(method='GET', path=path)

    def get_training_job(
            self, organization_id: str, job_definition_name: str, training_job_id: str) -> dict:
        """get a training job

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>/jobs/<training_job_id>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                training_job_id = "1443722127663"
                response = api_client.get_training_job(organization_id, job_definition_name, training_job_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **training_job_id** (str): TRAINING_JOB_ID

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "job_definition_id": "1443714239154",
                    "user_parameters": {},
                    "start_time": null,
                    "created_at": "2018-05-17T12:43:59.322367Z",
                    "job_definition_version": 1,
                    "completion_time": null,
                    "status": "Pending",
                    "modified_at": "2018-05-17T12:43:59.322673Z",
                    "training_job_id": "1443722127663",
                    "creator": {
                        "email": "test@abeja.asia",
                        "is_registered": true,
                        "created_at": "2017-05-26T01:38:46Z",
                        "id": "1128347408389",
                        "display_name": null,
                        "updated_at": "2018-01-04T03:02:12Z",
                        "role": "admin"
                    },
                    "description": null,
                    "statistics": null
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/jobs/{}'.format(
            organization_id, job_definition_name, training_job_id)
        return self._connection.api_request(method='GET', path=path)

    def delete_training_job(
            self, organization_id: str, job_definition_name: str, training_job_id: str) -> dict:
        """delete a training job

        API reference: DELETE /organizations/<organization_id>/training/definitions/<job_definition_name>/jobs/<training_job_id>

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                training_job_id = "1443722127663"
                response = api_client.delete_training_job_definition_version(organization_id,
                                                                             job_definition_name,
                                                                             training_job_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **training_job_id** (str): TRAINING_JOB_ID

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "message": "test_job_definition:1443722127663 deleted"
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/jobs/{}'.format(
            organization_id, job_definition_name, training_job_id)
        return self._connection.api_request(method='DELETE', path=path)

    def get_training_result(
            self, organization_id: str, job_definition_name: str, training_job_id: str) -> dict:
        """get a training job result

        API reference: GET /organizations/<organization_id>/training/definitions/<job_definition_name>/jobs/<training_job_id>/result

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                job_definition_name = "test_job_definition"
                training_job_id = "1443722127663"
                response = api_client.get_training_result(organization_id, job_definition_name,
                                                          training_job_id)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **job_definition_name** (str): training job definition name
            - **training_job_id** (str): TRAINING_JOB_ID

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "artifacts": {
                        "complete": {
                            "uri": "dummy_url",
                        }
                    }
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/training/definitions/{}/jobs/{}/result'.format(
            organization_id, job_definition_name, training_job_id)
        return self._connection.api_request(method='GET', path=path)

    def update_statistics(self, organization_id: str, job_definition_name: str, training_job_id: str,
                          statistics: dict) -> dict:
        """update a training job statistics

        API reference: POST /organizations/<organization_id>/training/definitions/<job_definition_name>/jobs/<training_job_id>/statistics

        Request Syntax:
            .. code-block:: python

                from abeja.train.statistics import Statistics

                statistics = Statistics(progress_percentage=0.5, epoch=1, num_epochs=5, key1='value1')
                statistics.add_stage(name=Statistics.STAGE_TRAIN, accuracy=0.9, loss=0.05)
                statistics.add_stage(name=Statistics.STAGE_VALIDATION, accuracy=0.8, loss=0.1, key2=2)

                response = api_client.update_statistics(statistics)

        Params:
            - **statistics** (str): statistics needs to be saved and updated

        Return type:
            dict

        Returns:
            Response Syntax:

            .. code-block:: json

                {
                  "statistics": {
                    "num_epochs": 5,
                    "epoch": 1,
                    "progress_percentage": 0.5,
                    "stages": {
                      "train": {
                        "accuracy": 0.9,
                        "loss": 0.05
                      },
                      "validation": {
                        "accuracy": 0.8,
                        "loss": 0.1,
                        "key2": 2
                      }
                    },
                    "key1": "value1"
                  }
                }

        Raises:
            - BadRequest
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        data = {
            'statistics': statistics
        }
        path = '/organizations/{}/training/definitions/{}/jobs/{}/statistics'.format(
            organization_id, job_definition_name, training_job_id)
        return self._connection.api_request(method='POST', path=path, json=data)
